# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import regex as re
import sre_constants

from prettyqt import gui


class RegexPatternValidator(gui.Validator):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.error_message = ""
        self.compiled = None

    def __repr__(self):
        return "RegexPatternValidator()"

    def validate(self, text, pos=0):
        # if text == "":
        #     self.compiled = None
        #     return (self.Intermediate, text, pos)
        try:
            self.compiled = re.compile(text)
        except sre_constants.error as e:
            self.error_message = str(e)
            self.compiled = None
            return (self.Intermediate, text, pos)
        except re._regex_core.error as e:
            self.error_message = str(e)
            self.compiled = None
            return (self.Intermediate, text, pos)
        else:
            self.error_message = ""
            return (self.Acceptable, text, pos)


if __name__ == "__main__":
    from prettyqt import widgets
    app = widgets.app()
    w = widgets.LineEdit()
    val = RegexPatternValidator()
    w.set_validator(val)
    w.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from contextlib import contextmanager
from typing import Callable, Dict, Optional
import operator
import functools

from qtpy import QtCore, QtGui, QtWidgets

from prettyqt import core, gui, widgets
from prettyqt.utils import bidict

POLICIES = bidict(none=QtCore.Qt.NoContextMenu,
                  prevent=QtCore.Qt.PreventContextMenu,
                  default=QtCore.Qt.DefaultContextMenu,
                  actions=QtCore.Qt.ActionsContextMenu,
                  custom=QtCore.Qt.CustomContextMenu,
                  showhide_menu="showhide_menu")

MODALITIES = bidict(window=QtCore.Qt.WindowModal,
                    application=QtCore.Qt.ApplicationModal,
                    none=QtCore.Qt.NonModal)

CURSOR_SHAPES = bidict(arrow=QtCore.Qt.ArrowCursor,
                       uparrow=QtCore.Qt.UpArrowCursor,
                       cross=QtCore.Qt.CrossCursor,
                       wait=QtCore.Qt.WaitCursor,
                       caret=QtCore.Qt.IBeamCursor,
                       size_vertical=QtCore.Qt.SizeVerCursor,
                       size_horizonal=QtCore.Qt.SizeHorCursor,
                       size_topright=QtCore.Qt.SizeBDiagCursor,
                       size_topleft=QtCore.Qt.SizeFDiagCursor,
                       size_all=QtCore.Qt.SizeAllCursor,
                       blank=QtCore.Qt.BlankCursor,
                       split_vertical=QtCore.Qt.SplitVCursor,
                       split_horizontal=QtCore.Qt.SplitHCursor,
                       pointing_hand=QtCore.Qt.PointingHandCursor,
                       forbidden=QtCore.Qt.ForbiddenCursor,
                       open_hand=QtCore.Qt.OpenHandCursor,
                       closed_hand=QtCore.Qt.ClosedHandCursor,
                       whats_this=QtCore.Qt.WhatsThisCursor,
                       busy=QtCore.Qt.BusyCursor,
                       drag_move=QtCore.Qt.DragMoveCursor,
                       drag_copy=QtCore.Qt.DragCopyCursor,
                       drag_link=QtCore.Qt.DragLinkCursor,
                       bitmap=QtCore.Qt.BitmapCursor)

FOCUS_POLICIES = bidict(tab=QtCore.Qt.TabFocus,
                        click=QtCore.Qt.ClickFocus,
                        strong=QtCore.Qt.StrongFocus,
                        wheel=QtCore.Qt.WheelFocus,
                        none=QtCore.Qt.NoFocus)

WINDOW_FLAGS = bidict(frameless=QtCore.Qt.FramelessWindowHint,
                      popup=QtCore.Qt.Popup,
                      stay_on_top=QtCore.Qt.WindowStaysOnTopHint,
                      tool=QtCore.Qt.Tool,
                      window_title=QtCore.Qt.WindowTitleHint,
                      customize_window=QtCore.Qt.CustomizeWindowHint)

QtWidgets.QWidget.__bases__ = (core.Object, QtGui.QPaintDevice)


class Widget(QtWidgets.QWidget):

    def __repr__(self):
        return f"{self.__class__.__name__}: {self.__getstate__()}"

    def __getstate__(self):
        return dict(layout=self.layout(),
                    size_policy=self.get_size_policy(),
                    accessible_name=self.accessibleName(),
                    tooltip=self.toolTip(),
                    statustip=self.statusTip())

    def __setstate__(self, state):
        self.__init__()
        self.set_layout(state["layout"])
        self.setSizePolicy(state["size_policy"])
        self.setAccessibleName(state["accessible_name"])
        self.setToolTip(state.get("tooltip", ""))
        self.setStatusTip(state.get("statustip", ""))

    def resize(self, *size):
        if isinstance(size[0], tuple):
            super().resize(*size[0])
        else:
            super().resize(*size)

    def set_min_size(self, *size):
        self.setMinimumSize(*size)

    def set_max_size(self, *size):
        self.setMaximumSize(*size)

    def set_min_width(self, width: int):
        self.setMinimumWidth(width)

    def set_max_width(self, width: int):
        self.setMaximumWidth(width)

    def set_min_height(self, height: int):
        self.setMinimumHeight(height)

    def set_max_height(self, height: int):
        self.setMaximumHeight(height)

    @property
    def title(self) -> str:
        return self.windowTitle()

    @title.setter
    def title(self, name: str):
        self.setWindowTitle(name)

    @property
    def enabled(self) -> bool:
        return self.isEnabled()

    @enabled.setter
    def enabled(self, state: bool):
        self.setEnabled(state)

    def set_enabled(self, enabled: bool = True):
        self.setEnabled(enabled)

    def set_disabled(self):
        self.setEnabled(False)

    def set_title(self, title: str):
        self.setWindowTitle(title)

    def set_tooltip(self, text: str):
        self.setToolTip(text)

    def set_font(self,
                 font_name: Optional[str] = None,
                 font_size: int = -1,
                 weight: int = -1,
                 italic: bool = False) -> gui.Font:
        if font_name is None:
            font_name = self.font().family()
        font = gui.Font(font_name, font_size, weight, italic)
        self.setFont(font)
        return font

    def set_window_flags(self, *flags: str, append: bool = False):
        for flag in flags:
            if flag not in WINDOW_FLAGS:
                raise ValueError("Invalid window flag")
        flags = functools.reduce(operator.ior, [WINDOW_FLAGS[t] for t in flags])
        if append:
            flags = flags | self.windowFlags()
        self.setWindowFlags(flags)

    def set_flags(self,
                  minimize: Optional[bool] = None,
                  maximize: Optional[bool] = None,
                  close: Optional[bool] = None,
                  stay_on_top: Optional[bool] = None,
                  frameless: Optional[bool] = None,
                  window: Optional[bool] = None):
        if minimize is not None:
            self.setWindowFlag(QtCore.Qt.WindowMinimizeButtonHint, minimize)
        if maximize is not None:
            self.setWindowFlag(QtCore.Qt.WindowMaximizeButtonHint, maximize)
        if close is not None:
            self.setWindowFlag(QtCore.Qt.WindowCloseButtonHint, close)
        if stay_on_top is not None:
            self.setWindowFlag(QtCore.Qt.WindowStaysOnTopHint, stay_on_top)
        if frameless is not None:
            self.setWindowFlag(QtCore.Qt.FramelessWindowHint, frameless)
        if window is not None:
            self.setWindowFlag(QtCore.Qt.Window, window)

    def set_modality(self, modality: str = "window"):
        """set modality for the dialog

        Valid values for modality: "modeless", "window", "application"

        Args:
            modality: modality for the main window (default: {"window"})

        Raises:
            ValueError: modality type does not exist
        """
        if modality not in MODALITIES:
            raise ValueError("Invalid value for modality.")
        self.setWindowModality(MODALITIES[modality])

    def get_modality(self) -> str:
        """get the current modality modes as a string

        Possible values: "modeless", "window", "application"

        Returns:
            modality mode
            str
        """
        return MODALITIES.inv[self.windowModality()]

    @contextmanager
    def updates_off(self):
        self.setUpdatesEnabled(False)
        yield None
        self.setUpdatesEnabled(True)

    def set_size_policy(self,
                        horizontal: Optional[str] = None,
                        vertical: Optional[str] = None):
        """sets the sizes policy

        possible values for both parameters are "fixed", "minimum", "maximum",
        "preferred", "expanding", "minimum_expanding" and "ignored"

        Args:
            horizontal: horizontal size policy
            vertical: vertical size policy
        """
        sp = self.get_size_policy()
        if horizontal is not None:
            sp.set_horizontal_policy(horizontal)
        if vertical is not None:
            sp.set_vertical_policy(vertical)
        self.setSizePolicy(sp)

    def get_size_policy(self):
        qpol = self.sizePolicy()
        if isinstance(qpol, widgets.SizePolicy):
            return qpol
        pol = widgets.SizePolicy(qpol.horizontalPolicy(),
                                 qpol.verticalPolicy(),
                                 qpol.controlType())
        pol.setHeightForWidth(qpol.hasHeightForWidth())
        pol.setWidthForHeight(qpol.hasWidthForHeight())
        pol.setHorizontalStretch(qpol.horizontalStretch())
        pol.setVerticalStretch(qpol.verticalStretch())
        return pol

    def set_background_color(self, color):
        self.setStyleSheet(f"background-color: {color};")

    def set_stylesheet(self, item, dct: Dict[str, str]) -> str:
        ss = "; ".join(f"{k.replace('_', '-')}: {v}" for k, v in dct.items())
        stylesheet = f"{item} {{{ss};}}"
        self.setStyleSheet(stylesheet)
        return stylesheet

    def set_contextmenu_policy(self, policy: str):
        """set contextmenu policy for given item view

        Allowed values are "none", "prevent", "default", "actions",
                           "custom", "showhide_menu"

        Args:
            policy: contextmenu policy to use

        Raises:
            ValueError: policy does not exist
        """
        if policy not in POLICIES:
            raise ValueError("invalid selection behaviour")
        self.setContextMenuPolicy(POLICIES[policy])

    def get_contextmenu_policy(self) -> str:
        """returns current contextmenu policy

        Possible values: "none", "prevent", "default", "actions",
                         "custom", "showhide_menu"

        Returns:
            contextmenu policy
        """
        return POLICIES.inv[self.contextMenuPolicy()]

    def set_custom_menu(self, method: Callable):
        self.set_contextmenu_policy("custom")
        self.customContextMenuRequested.connect(method)

    def set_layout(self, layout):
        if layout in ["horizontal", "vertical"]:
            self.box = widgets.BoxLayout(layout)
        elif layout == "grid":
            self.box = widgets.GridLayout()
        elif layout == "form":
            self.box = widgets.FormLayout()
        elif layout == "stacked":
            self.box = widgets.StackedLayout()
        elif layout == "flow":
            from prettyqt import custom_widgets
            self.box = custom_widgets.FlowLayout()
        else:
            self.box = layout
        if self.box is not None:
            self.setLayout(self.box)

    def center(self):
        qr = self.frameGeometry()
        cp = gui.GuiApplication.screens()[0].geometry().center()
        qr.moveCenter(cp)
        self.move(qr.topLeft())

    def raise_dock(self) -> bool:
        node = self
        while node:
            node = node.parent()
            if isinstance(node, QtWidgets.QDockWidget):
                node.setVisible(True)
                node.raise_()
                return True
        return False

    def set_cursor(self, cursor: str):
        if cursor not in CURSOR_SHAPES:
            raise ValueError(f"Invalid cursor '{cursor}'. "
                             f"Valid values: {CURSOR_SHAPES.keys()}")
        self.setCursor(CURSOR_SHAPES[cursor])

    def set_focus_policy(self, policy: str):
        if policy not in FOCUS_POLICIES:
            raise ValueError(f"Invalid policy '{policy}'. "
                             f"Valid values: {FOCUS_POLICIES.keys()}")
        self.setFocusPolicy(FOCUS_POLICIES[policy])

    def get_focus_policy(self) -> str:
        return FOCUS_POLICIES.inv[self.focusPolicy()]

    def set_font_size(self, size: int):
        font = self.font()
        font.setPointSize(size)
        self.setFont(font)


if __name__ == "__main__":
    app = QtWidgets.QApplication([])
    widget = Widget()
    widget.show()
    app.exec_()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `prettyqt` package."""

import pickle

from qtpy import QtCore

from prettyqt import charts


def test_chart(qtbot):
    chart = charts.Chart(parent=None)
    chart.hide_legend()
    chart.show_legend()
    chart.set_legend_alignment("right")
    chart.set_theme("Dark")
    chart.set_animation_options("series")


def test_chartview(qtbot):
    widget = charts.ChartView(parent=None)
    widget.get_image()
    qtbot.addWidget(widget)
    qtbot.keyPress(widget, QtCore.Qt.Key_F11)
    # qtbot.keyPress(widget, QtCore.Qt.Key_Minus)
    # qtbot.keyPress(widget, QtCore.Qt.Key_Plus)
    qtbot.keyPress(widget, QtCore.Qt.Key_Left)
    qtbot.keyPress(widget, QtCore.Qt.Key_Right)
    qtbot.keyPress(widget, QtCore.Qt.Key_Up)
    qtbot.keyPress(widget, QtCore.Qt.Key_Down)
    qtbot.mousePress(widget, QtCore.Qt.RightButton)
    qtbot.mouseMove(widget, delay=100)


def test_lineseries(qtbot):
    line = charts.LineSeries()
    with open("data.pkl", "wb") as jar:
        pickle.dump(line, jar)
    with open("data.pkl", "rb") as jar:
        line = pickle.load(jar)


def test_scatterseries(qtbot):
    charts.ScatterSeries()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `prettyqt` package."""

from prettyqt import custom_models, widgets


def test_transposeproxymodel():
    source = widgets.FileSystemModel()
    model = custom_models.TransposeProxyModel(source)
    idx = model.index(0, 0)
    model.data(idx)
    model.columnCount()
    model.rowCount()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `datacook` package."""

# import pytest

import pytest

from prettyqt import widgets


@pytest.fixture(scope="session")
def qapp():
    app = widgets.Application([])
    app.set_metadata(app_name="test",
                     app_version="1.0.0",
                     org_name="test",
                     org_domain="test")
    yield app



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets, QtCore

from prettyqt import core, widgets
from prettyqt.utils import bidict


STATES = bidict(unchecked=QtCore.Qt.Unchecked,
                partial=QtCore.Qt.PartiallyChecked,
                checked=QtCore.Qt.Checked)


QtWidgets.QCheckBox.__bases__ = (widgets.AbstractButton,)


class CheckBox(QtWidgets.QCheckBox):

    value_changed = core.Signal(int)

    def __init__(self, label="", parent=None, checked=False):
        super().__init__(label, parent)
        self.stateChanged.connect(self.value_changed)
        self.setChecked(checked)

    def __getstate__(self):
        return dict(object_name=self.id,
                    checkable=self.isCheckable(),
                    checkstate=self.get_checkstate(),
                    tooltip=self.toolTip(),
                    statustip=self.statusTip(),
                    is_tristate=self.isTristate(),
                    text=self.text(),
                    enabled=self.isEnabled())

    def __setstate__(self, state):
        self.__init__()
        self.id = state.get("object_name", "")
        self.setCheckable(state["checkable"])
        self.setTristate(state.get("is_tristate", False))
        self.set_checkstate(state["checkstate"])
        self.setText(state.get("text", ""))
        self.setEnabled(state.get("enabled", True))
        self.setToolTip(state.get("tooltip", ""))
        self.setStatusTip(state.get("statustip", ""))

    def __bool__(self):
        return self.isChecked()

    @property
    def is_on(self) -> bool:
        return self.isChecked()

    @is_on.setter
    def is_on(self, state: bool):
        self.setChecked(state)

    def set_checkstate(self, state: str):
        """set checkstate of the checkbox

        valid values are: unchecked, partial, checked

        Args:
            state: checkstate to use

        Raises:
            ValueError: invalid checkstate
        """
        if state not in STATES:
            raise ValueError("Invalid checkstate.")
        self.setCheckState(STATES[state])

    def get_checkstate(self) -> bool:
        """returns checkstate

        possible values are "unchecked", "partial", "checked"

        Returns:
            checkstate
        """
        return STATES.inv[self.checkState()]

    def get_value(self) -> bool:
        return self.isChecked()

    def set_value(self, value: bool):
        self.setChecked(value)


if __name__ == "__main__":
    app = widgets.app()
    widget = CheckBox("test")
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from prettyqt import core, gui, syntaxhighlighters

KEYWORDS = [
    "and", "assert", "break", "continue", "class", "def"
    "del", "elif", "else", "except", "exec", "finally",
    "for", "from", "global", "if", "import", "in",
    "is", "lambda", "not", "or", "pass", "print",
    "raise", "return", "try", "while", "yield",
    "None", "True", "False",
]

OPERATORS = [
    "=",
    "==", "!=", "<", "<=", ">", ">=",
    r"\+", "-", r"\*", "/", "//", r"\%", r"\*\*",
    r"\+=", "-=", r"\*=", "/=", r"\%=",
    r"\^", r"\|", r"\&", r"\~", ">>", "<<",
]


class Rule(syntaxhighlighters.HighlightRule):
    pass


class Keyword(Rule):
    regex = r"\b%s\b" % r"\b|\b".join(KEYWORDS)
    color = gui.Color(200, 120, 50)
    bold = True


class Operator(Rule):
    regex = r"%s" % "|".join(OPERATORS)
    color = gui.Color(150, 150, 150)


class Bracket(Rule):
    regex = r"[\[\]\{\}\(\)]"
    color = "darkgray"


class Self(Rule):
    regex = r"\bself\b"
    color = gui.Color(150, 85, 140)
    italic = True


class String(Rule):
    regex = [r'"[^"\\]*(\\.[^"\\]*)*"', r"'[^'\\]*(\\.[^'\\]*)*'"]
    color = gui.Color(20, 110, 100)


class Def(Rule):
    regex = r"\bdef\b\s*(\w+)"
    color = gui.Color(20, 20, 255)
    bold = True
    nth = 1


class Class(Rule):
    regex = r"\bclass\b\s*(\w+)"
    color = gui.Color(20, 20, 255)
    bold = True
    nth = 1


class Comment(Rule):
    regex = r"#[^\n]*"
    color = "lightgray"


class Number(Rule):
    regex = [r"\b[+-]?[0-9]+[lL]?\b",
             r"\b[+-]?0[xX][0-9A-Fa-f]+[lL]?\b",
             r"\b[+-]?[0-9]+(?:\.[0-9]+)?(?:[eE][+-]?[0-9]+)?\b"]
    color = gui.Color(100, 150, 190)


# Multi-line strings (expression, flag, style)
# FIXME: The triple-quotes in these two lines will mess up the
# syntax highlighting from this point onward
fmt = gui.TextCharFormat()
fmt.set_foreground_color([30, 120, 110])
TRI_SINGLE = (core.RegExp("'''"), 1, fmt)
TRI_DOUBLE = (core.RegExp('"""'), 2, fmt)


class PythonHighlighter(gui.SyntaxHighlighter):
    """Syntax highlighter for the Python language.
    """
    RULES = Rule.__subclasses__()

    def highlightBlock(self, text):
        """Apply syntax highlighting to the given block of text.
        """
        # Do other syntax formatting
        super().highlightBlock(text)
        self.setCurrentBlockState(0)
        # Do multi-line strings
        if not self.match_multiline(text, *TRI_SINGLE):
            self.match_multiline(text, *TRI_DOUBLE)

    def match_multiline(self, text, delimiter, in_state, style):
        """Do highlighting of multi-line strings. ``delimiter`` should be a
        ``core.RegExp`` for triple-single-quotes or triple-double-quotes, and
        ``in_state`` should be a unique integer to represent the corresponding
        state changes when inside those strings. Returns True if we're still
        inside a multi-line string when this function is finished.
        """
        # If inside triple-single quotes, start at 0
        if self.previousBlockState() == in_state:
            start = 0
            add = 0
        # Otherwise, look for the delimiter on this line
        else:
            start = delimiter.indexIn(text)
            # Move past this match
            add = delimiter.matchedLength()

        # As long as there's a delimiter match on this line...
        while start >= 0:
            # Look for the ending delimiter
            end = delimiter.indexIn(text, start + add)
            # Ending delimiter on this line?
            if end >= add:
                length = end - start + add + delimiter.matchedLength()
                self.setCurrentBlockState(0)
            # No; multi-line string
            else:
                self.setCurrentBlockState(in_state)
                length = len(text) - start + add
            # Apply formatting
            self.setFormat(start, length, style)
            # Look for the next match
            start = delimiter.indexIn(text, start + length)

        # Return True if still inside a multi-line string, False otherwise
        return self.currentBlockState() == in_state


if __name__ == "__main__":
    from prettyqt import widgets
    app = widgets.app()
    editor = widgets.PlainTextEdit()
    highlighter = PythonHighlighter(editor.document())
    editor.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtGui


class TextBlockUserData(QtGui.QTextBlockUserData):
    """ Storage for the user data associated with each line.
    """

    def __init__(self, **kwds):
        for key, value in kwds.items():
            setattr(self, key, value)
        super().__init__()

    def __repr__(self):
        attrs = [i for i in dir(self) if not i.startswith('__')]
        kwds = ", ".join("%s=%r" % (attr, getattr(self, attr)) for attr in attrs)
        return f"{self.__class__.__name__}({kwds})"



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import pathlib
from typing import Optional

from prettyqt import core, widgets


class FileChooserButton(widgets.Widget):

    value_changed = core.Signal(pathlib.Path)

    def __init__(self, extensions=None, mode="save", file_mode="existing_files",
                 root=None, parent=None):
        """initialize FileChooserButton

        Args:
            extensions: dict allowed extensions (default: {None})
                        form: "'name': ['.ext1', '.ext2']"
            mode: Accept mode ("save" or "load") (default: {"save"})
            file_mode: File mode ("existing_files", "existing_file", "any_file",
                                  or "directory") (default: {"existing_files"})
            parent: parent widget (default: {None})
        """
        super().__init__(parent)
        self.path = None
        self.extensions = extensions
        self.mode = mode
        self.file_mode = file_mode
        self.root = root
        layout = widgets.BoxLayout("horizontal", self)
        layout.set_margin(0)
        self.lineedit = widgets.LineEdit()
        self.lineedit.set_read_only()
        layout += self.lineedit
        action = widgets.Action()
        if self.file_mode == "directory":
            action.set_icon("mdi.folder-outline")
        else:
            action.set_icon("mdi.file-outline")
        action.triggered.connect(self.open_file)

        self.button = widgets.ToolButton()
        self.button.setDefaultAction(action)
        layout += self.button

    def __getstate__(self):
        return dict(path=self.path,
                    extensions=self.extensions,
                    enabled=self.isEnabled())

    def __setstate__(self, state):
        self.__init__(state["extensions"])
        self.set_path(state["path"])
        self.set_enabled(state.get("enabled", True))

    @core.Slot()
    def open_file(self):
        dialog = widgets.FileDialog(parent=self,
                                    path_id="file_path",
                                    mode=self.mode,
                                    path=self.root,
                                    file_mode=self.file_mode)
        if self.extensions:
            dialog.set_extension_filter(self.extensions)
        if not dialog.choose():
            return None
        self.set_path(dialog.selected_file())
        self.value_changed.emit(self.path)

    def set_path(self, path):
        self.path = path
        self.lineedit.set_text(str(path))

    def get_value(self) -> Optional[pathlib.Path]:
        return self.path

    def set_value(self, value):
        self.set_path(value)


if __name__ == "__main__":
    app = widgets.app()
    btn = FileChooserButton()
    btn.show()
    btn.value_changed.connect(print)
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets

from prettyqt import widgets

QtWidgets.QWidgetAction.__bases__ = (widgets.Action,)


class WidgetAction(QtWidgets.QWidgetAction):
    def __init__(self, parent=None):
        super().__init__(parent)
    #     self.set_text(text)
    #     self.set_icon(icon)
    #     self.set_shortcut(shortcut)
    #     self.set_tooltip(tooltip)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from typing import Union

import contextlib

from qtpy import QtGui

from prettyqt.utils import bidict


MOVE_MODES = bidict(move=QtGui.QTextCursor.MoveAnchor,
                    keep=QtGui.QTextCursor.KeepAnchor)

MOVE_OPERATIONS = bidict(no_move=QtGui.QTextCursor.NoMove,
                         start=QtGui.QTextCursor.Start,
                         start_of_line=QtGui.QTextCursor.StartOfLine,
                         start_of_block=QtGui.QTextCursor.StartOfBlock,
                         start_of_word=QtGui.QTextCursor.StartOfWord,
                         previous_block=QtGui.QTextCursor.PreviousBlock,
                         previous_char=QtGui.QTextCursor.PreviousCharacter,
                         previous_word=QtGui.QTextCursor.PreviousWord,
                         up=QtGui.QTextCursor.Up,
                         left=QtGui.QTextCursor.Left,
                         word_left=QtGui.QTextCursor.WordLeft,
                         end=QtGui.QTextCursor.End,
                         end_of_line=QtGui.QTextCursor.EndOfLine,
                         end_of_word=QtGui.QTextCursor.EndOfWord,
                         end_of_block=QtGui.QTextCursor.EndOfBlock,
                         next_block=QtGui.QTextCursor.NextBlock,
                         next_char=QtGui.QTextCursor.NextCharacter,
                         next_word=QtGui.QTextCursor.NextWord,
                         down=QtGui.QTextCursor.Down,
                         right=QtGui.QTextCursor.Right,
                         word_right=QtGui.QTextCursor.WordRight,
                         next_cell=QtGui.QTextCursor.NextCell,
                         previous_cell=QtGui.QTextCursor.PreviousCell,
                         next_row=QtGui.QTextCursor.NextRow,
                         previous_row=QtGui.QTextCursor.PreviousRow)

SELECTION_TYPES = bidict(document=QtGui.QTextCursor.Document,
                         block_under_cursor=QtGui.QTextCursor.BlockUnderCursor,
                         line_under_cursor=QtGui.QTextCursor.LineUnderCursor,
                         word_under_cursor=QtGui.QTextCursor.WordUnderCursor)


class TextCursor(QtGui.QTextCursor):

    def move_position(self, operation: str, mode: str = "move", n: int = 1) -> bool:
        op = MOVE_OPERATIONS[operation]
        mode = MOVE_MODES[mode]
        return self.movePosition(op, mode, n)

    def set_position(self, pos: int, mode: str = "move"):
        """set cursor to given position

        Args:
            pos: Cursor position
            mode: Move mode (default: {"move"})
        """
        self.setPosition(pos, MOVE_MODES[mode])

    def select(self, selection):
        if selection in SELECTION_TYPES:
            selection = SELECTION_TYPES[selection]
        super().select(selection)

    def span(self) -> tuple:
        return (self.anchor(), self.position())

    def select_text(self, start_pos: Union[int, str], end_pos: Union[int, str]) -> str:
        """select text from start position to end position.

        Positions can be either an integer index or a move operation

        Args:
            start_pos: Start position
            end_pos: End position
        """
        if isinstance(start_pos, int):
            self.set_position(start_pos)
        else:
            self.move_position(start_pos)
        if isinstance(end_pos, int):
            self.set_position(end_pos, mode="keep")
        else:
            self.move_position(end_pos, mode="keep")
        return self.selectedText()

    def replace_text(self,
                     start_pos: int,
                     end_pos: Union[str, int],
                     to_replace: str):
        if isinstance(start_pos, int):
            self.set_position(start_pos)
        else:
            self.move_position(start_pos)
        if isinstance(end_pos, int):
            self.set_position(end_pos, mode="keep")
        else:
            self.move_position(end_pos, mode="keep")
        self.insertText(to_replace)
        self.select_text(start_pos, start_pos + len(to_replace))

    @contextlib.contextmanager
    def edit_block(self):
        self.beginEditBlock()
        yield
        self.endEditBlock()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff

for full list, see:
- https://cdn.materialdesignicons.com/3.0.39/
"""

import logging
from prettyqt import core


class LineSignalLogger(logging.Handler, core.Object):
    log_line = core.Signal(str)

    def __init__(self):
        super().__init__()
        core.Object.__init__(self)

    def emit(self, record):
        msg = self.format(record)
        self.log_line.emit(msg)


class RecordSignalLogger(logging.Handler, core.Object):
    log_record = core.Signal(logging.LogRecord)

    def __init__(self):
        super().__init__()
        core.Object.__init__(self)

    def emit(self, record):
        self.log_record.emit(record)



# -*- coding: utf-8 -*-

from qtpy import QtCore

from prettyqt import core


class TransposeProxyModel(core.AbstractProxyModel):

    def __init__(self, source_model):
        super().__init__()
        self._source_model = None
        self.setSourceModel(source_model)

    def setSourceModel(self, source_model):
        self._source_model = source_model
        super().setSourceModel(source_model)

    def mapFromSource(self, source_index):
        return self.index(source_index.column(), source_index.row())

    def mapToSource(self, proxy_index):
        return self._source_model.index(proxy_index.column(), proxy_index.row())

    def index(self, row: int, column: int, parent=None) -> QtCore.QModelIndex:
        return self.createIndex(row, column)

    def parent(self, index):
        return None

    def rowCount(self, parent=core.ModelIndex()) -> int:
        return self._source_model.columnCount(parent)

    def columnCount(self, parent=core.ModelIndex()) -> int:
        return self._source_model.rowCount(parent)

    def data(self, index, role=QtCore.Qt.DisplayRole):
        if not index.isValid():
            return False
        return self._source_model.data(self.mapFromSource(index), role)

    def headerData(self, section: int, orientation, role):
        return self._source_model.headerData(section, orientation, role)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from prettyqt import widgets


class BoolDictToolButton(widgets.ToolButton):

    def __init__(self, title, icon=None, dct=None, parent=None):
        super().__init__(parent=parent)
        self.set_text(title)
        self.set_icon(icon)
        self.setMenu(widgets.Menu())
        self.set_popup_mode("instant")
        if dct:
            self.set_dict(dct)

    def __getitem__(self, key):
        menu = self.menu()
        return menu[key].isChecked()

    def __setitem__(self, key, value):
        menu = self.menu()
        menu[key].setChecked(value)

    def set_dict(self, dct):
        menu = self.menu()
        for k, v in dct.items():
            action = widgets.Action()
            action.set_text(v)
            action.setCheckable(True)
            action.id = k
            menu.add(action)

    def as_dict(self):
        return {act.id: act.isChecked() for act in self.menu()}


if __name__ == "__main__":
    app = widgets.app()
    dct = dict(a="test",
               b="test2")
    w = BoolDictToolButton("Title", None, dct)
    w.show()
    app.exec_()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `prettyqt` package."""


def test_application(qapp):
    qapp.set_icon("mdi.timer")
    qapp.set_icon(None)
    qapp.set_metadata(app_name="test",
                      app_version="1.0.0",
                      org_name="test",
                      org_domain="test")
    qapp.get_mainwindow()
    qapp.copy_to_clipboard("test")
    return True



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from typing import Union, Callable, Optional, Iterator

from qtpy import QtCore

from prettyqt import core
from prettyqt.utils import bidict

FLAGS = bidict(none=QtCore.QRegularExpression.NoPatternOption,
               ignorecase=QtCore.QRegularExpression.CaseInsensitiveOption,
               dotall=QtCore.QRegularExpression.DotMatchesEverythingOption,
               multiline=QtCore.QRegularExpression.MultilineOption,
               verbose=QtCore.QRegularExpression.ExtendedPatternSyntaxOption,
               inverted_greedyness=QtCore.QRegularExpression.InvertedGreedinessOption,
               dont_capture=QtCore.QRegularExpression.DontCaptureOption,
               unicode=QtCore.QRegularExpression.UseUnicodePropertiesOption)

MATCH_TYPES = bidict(normal=QtCore.QRegularExpression.NormalMatch,
                     prefer_complete=QtCore.QRegularExpression.PartialPreferCompleteMatch,
                     prefer_first=QtCore.QRegularExpression.PartialPreferFirstMatch,
                     no_match=QtCore.QRegularExpression.NoMatch)

MATCH_OPTIONS = bidict(none=QtCore.QRegularExpression.NoMatchOption,
                       anchored=QtCore.QRegularExpression.AnchoredMatchOption)


class RegularExpression(QtCore.QRegularExpression):

    def __init__(self, pattern="", flags=FLAGS["none"]):
        if isinstance(pattern, QtCore.QRegularExpression):
            super().__init__(pattern)
        else:
            if isinstance(flags, int):
                flags = core.RegularExpression.PatternOptions(flags)
            super().__init__(pattern, flags)

    def __repr__(self):
        return f"RegularExpression({self.pattern()!r})"

    def __reduce__(self):
        return (self.__class__, (self.pattern(), int(self.flags)))

    def globalMatch(self, *args, **kwargs) -> core.RegularExpressionMatchIterator:
        it = super().globalMatch(*args, **kwargs)
        return core.RegularExpressionMatchIterator(it)

    def global_match(self,
                     text: str,
                     offset: int = 0,
                     match_type: str = "normal",
                     anchored: bool = False):
        options = MATCH_OPTIONS["anchored"] if anchored else MATCH_OPTIONS["none"]
        return self.globalMatch(text, offset, MATCH_TYPES[match_type], options)

    def match(self,
              text: str,
              offset: int = 0,
              match_type: str = "normal",
              anchored: bool = False):
        if isinstance(match_type, str):
            match_type = MATCH_TYPES[match_type]
        if isinstance(anchored, bool):
            options = MATCH_OPTIONS["anchored"] if anchored else MATCH_OPTIONS["none"]
        else:
            options = anchored
        match = super().match(text, offset, match_type, options)
        return core.RegularExpressionMatch(match)

    def fullmatch(self, string: str, pos: int = 0, endpos: Optional[int] = None):
        if endpos:
            string = string[:endpos]
        match = super().match(string, pos)
        if match.hasMatch() and len(string) == match.end() - match.start():
            return core.RegularExpressionMatch(match)
        else:
            return None

    def finditer(self,
                 string: str,
                 pos: int = 0,
                 endpos: Optional[int] = None) -> Iterator[core.RegularExpressionMatch]:
        for match in self.globalMatch(string[:endpos], offset=pos):
            match.pos = pos
            match.endpos = endpos
            match.string = string
            yield match

    def findall(self,
                string: str,
                pos: int = 0,
                endpos: Optional[int] = None) -> list:
        matches = [m for m in self.globalMatch(string[:endpos], offset=pos)]
        return [m.groups() if len(m.groups()) > 1 else m.group(0) for m in matches]

    def subn(self, repl: Union[str, Callable], string: str, count: int = 0):
        result = string
        matches = self.global_match(string)
        matches = list(matches)
        if count > 0:
            matches = matches[:count]
        matches = list(reversed(matches))
        for m in matches:
            to_replace = repl if isinstance(repl, str) else repl(m)
            for j in range(self.groups):
                to_replace = to_replace.replace(fr"\g<{j}>", m.group(j))
            for j in self.groupindex.keys():
                to_replace = to_replace.replace(fr"\g<{j}>", m.group(j))
            result = result[:m.start()] + to_replace + result[m.end():]
        return (result, min(len(matches), count))

    def sub(self, repl: Union[str, Callable], string: str, count: int = 0):
        res = self.subn(repl, string, count)
        return res[0]

    def search(self, string: str, pos: int = 0, endpos: Optional[int] = None):
        match = super().match(string[:endpos], pos)
        return match if match.hasMatch() else None

    def split(self, string: str, maxsplit: int = 0):
        result = list()
        matches = self.global_match(string)
        matches = list(matches)
        if 0 < maxsplit <= len(matches):
            remainder = string[matches[maxsplit - 1].end():]
            print(remainder)
        else:
            print(None)
            remainder = None
        if maxsplit > 0:
            matches = matches[:maxsplit]
        prev_match = None
        m = matches[0]
        if m.start() == 0:
            result.append("")
        else:
            result.append(string[0:m.start()])
        for g in m.groups():
            result.append(g)
        prev_match = m
        for m in matches[1:]:
            result.append(string[prev_match.end():m.start()])
            for g in m.groups():
                result.append(g)
            if m.end() == len(string):
                result.append("")
            prev_match = m
        if remainder:
            result.append(remainder)
        return result

    @property
    def groups(self):
        return self.captureCount()

    @property
    def groupindex(self):
        return {k: i for i, k in enumerate(self.namedCaptureGroups()[1:], start=1)}

    @property
    def flags(self):
        return self.patternOptions()


if __name__ == "__main__":
    reg = RegularExpression()
    reg.setPattern('-{1,2}')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `prettyqt` package."""

from prettyqt import webenginewidgets


def test_webengineview(qapp):
    widget = webenginewidgets.WebEngineView()
    widget.set_zoom(1.5)
    widget.set_url("http://www.google.de")
    widget.load_url("http://www.google.de")



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import functools
import pathlib
from typing import Optional

from prettyqt import custom_validators, custom_widgets, gui, widgets


class DataItem(object):
    def __init__(self,
                 label,
                 value=None,
                 optional=False,
                 include=True,
                 enabled_on=None,
                 disabled_on=None):
        self.set_value(value)
        self.enabled_on = [enabled_on] if enabled_on else list()
        self.disabled_on = [disabled_on] if disabled_on else list()
        self.name = None
        self.label = label
        self.optional = optional
        self.include = include
        self.colspan = 1
        self.label_col = 0
        self.is_enabled = True

    def __get__(self, instance, owner):
        return self.value

    def set_pos(self, col=0, colspan=None):
        """
        Set data item's position on a GUI layout
        """
        self.label_col = col
        self.colspan = colspan
        return self

    def set_value(self, value):
        self.value = value

    def store(self, prop):
        # self.set_prop("display", store=prop)
        return self

    def is_valid(self):
        return True

    def create_widget(self):
        widget = self._create_widget()
        if self.optional:
            return custom_widgets.OptionalWidget(widget, self.optional)
        return widget

    def _create_widget(self):
        return None


class Fixed(DataItem):
    pass


class Float(DataItem):
    """
    Construct a float data item
        * label [string]: name
        * value [float]: default value (optional)
        * min [float]: minimum value (optional)
        * max [float]: maximum value (optional)
        * slider [bool]: if True, shows a slider widget right after the line
          edit widget (default is False)
        * step [float]: step between tick values with a slider widget (optional)
        * unit [string]: physical unit (optional)
        * check [bool]: if False, value is not checked (optional, value=True)
    """

    def __init__(self, label, value=None, min_val=None, max_val=None,
                 unit="", step=0.1, slider=False, **kwargs):
        super().__init__(label, value=value, **kwargs)
        self.range = (min_val, max_val)
        self.step = step
        self.unit = unit

    def _create_widget(self):
        widget = widgets.DoubleSpinBox()
        widget.set_range(*self.range)
        widget.setSingleStep(self.step)
        widget.setSuffix(self.unit)
        if self.value is not None:
            widget.set_value(self.value)
        return widget


class Int(DataItem):
    """
    Construct an integer data item
        * label [string]: name
        * value [int]: default value (optional)
        * min [int]: minimum value (optional)
        * max [int]: maximum value (optional)
        * unit [string]: physical unit (optional)
        * slider [bool]: if True, shows a slider widget right after the line
          edit widget (default is False)
        * check [bool]: if False, value is not checked (optional, value=True)
    """

    def __init__(self, label, value=None, min_val=0, max_val=None,
                 unit="", step=1, slider=False, **kwargs):
        super().__init__(label, value=value, **kwargs)
        self.range = (min_val, max_val)
        self.step = step
        self.unit = unit
        self.slider = slider

    def _create_widget(self):
        if self.slider:
            widget = custom_widgets.InputAndSlider()
        else:
            widget = widgets.SpinBox()
            widget.setSuffix(self.unit)
        widget.set_range(*self.range)
        widget.set_step_size(self.step)
        if self.value is not None:
            widget.set_value(self.value)
        return widget


class Range(DataItem):
    """
    Construct a Range data item
        * label [string]: name
        * value [int]: default value (optional)
        * min [int]: minimum value (optional)
        * max [int]: maximum value (optional)
        * unit [string]: physical unit (optional)
        * slider [bool]: if True, shows a slider widget right after the line
          edit widget (default is False)
        * check [bool]: if False, value is not checked (optional, value=True)
    """

    def __init__(self,
                 label,
                 value=None,
                 min_val=0,
                 max_val=1,
                 **kwargs):
        super().__init__(label, value=value, **kwargs)
        self.range = (min_val, max_val)
        # self.step = step
        # self.unit = unit

    def _create_widget(self):
        widget = custom_widgets.SpanSliderWidget()
        # widget.setSuffix(self.unit)
        widget.set_range(*self.range)
        widget.set_step_size(self.step)
        if self.value is not None:
            widget.set_value(self.value)
        return widget


class String(DataItem):
    """
    Construct a string data item
        * label [string]: name
        * value [string]: default value (optional)
        * notempty [bool]: if True, empty string is not a valid value (opt.)
    """

    def __init__(self,
                 label,
                 value=None,
                 notempty=False,
                 regex=None,
                 **kwargs):
        super().__init__(label, value=value, **kwargs)
        self.notempty = notempty
        self.regex = regex

    def _create_widget(self):
        widget = widgets.LineEdit()
        if self.notempty:
            val = custom_validators.NotEmptyValidator()
            widget.set_validator(val)
        if self.regex is not None:
            val = gui.RegExpValidator()
            val.set_regex(self.regex)
            widget.set_validator(val)
        if self.value is not None:
            widget.set_value(self.value)
        return widget


class Code(DataItem):
    """
    Construct a code data item
        * label [string]: name
        * value [string]: default value (optional)
        * language [string]: language for syntax highlighting
    """

    def __init__(self,
                 label,
                 value=None,
                 language="python",
                 **kwargs):
        super().__init__(label, value=value, **kwargs)
        self.language = language

    def _create_widget(self):
        widget = custom_widgets.CodeEditor()
        if self.value is not None:
            widget.set_value(self.value)
        return widget


class Regex(DataItem):
    """
    Construct a code data item
        * label [string]: name
        * value [string]: default value (optional)
        * language [string]: language for syntax highlighting
    """

    def __init__(self,
                 label: str,
                 value=None,
                 show_flags: bool = True,
                 show_error: bool = False,
                 **kwargs):
        super().__init__(label, value=value, **kwargs)
        self.show_flags = show_flags
        self.show_error = show_error

    def _create_widget(self):
        widget = custom_widgets.RegexInput(show_flags=self.show_flags,
                                           show_error=self.show_error)
        if self.value is not None:
            widget.set_value(self.value)
        return widget


class IntList(DataItem):
    """
    Construct an IntList data item
        * label [string]: name
        * value [string]: default value (optional)
        * notempty [bool]: if True, empty string is not a valid value (opt.)
    """

    def __init__(self,
                 label,
                 value=None,
                 allow_single=False,
                 **kwargs):
        super().__init__(label, value=value, **kwargs)
        self.allow_single = allow_single

    def _create_widget(self):
        widget = custom_widgets.ListInput(allow_single=self.allow_single)
        if self.value is not None:
            widget.set_value(self.value)
        return widget


class FloatList(DataItem):
    """
    Construct an FloatList data item
        * label [string]: name
        * value [string]: default value (optional)
        * notempty [bool]: if True, empty string is not a valid value (opt.)
    """

    def __init__(self,
                 label,
                 value=None,
                 allow_single=False,
                 **kwargs):
        super().__init__(label, value=value, **kwargs)
        self.allow_single = allow_single

    def _create_widget(self):
        widget = custom_widgets.ListInput(allow_single=self.allow_single, typ=float)
        if self.value is not None:
            widget.set_value(self.value)
        return widget


class Bool(DataItem):
    """
    Construct a boolean data item
        * text [string]: form's field name (optional)
        * label [string]: name
        * value [string]: default value (optional)
        * check [bool]: if False, value is not checked (optional, value=True)
    """

    def __init__(self,
                 label,
                 value=False,
                 true_value=True,
                 false_value=False,
                 **kwargs):
        super().__init__(label, value=value, **kwargs)
        self.true_value = true_value
        self.false_value = false_value

    def _create_widget(self):
        widget = custom_widgets.MappedCheckBox(true_value=self.true_value,
                                               false_value=self.false_value)
        if self.value is not None:
            widget.set_value(self.value)
        return widget


class Color(DataItem):
    """
    Construct a color data item
        * label [string]: name
        * value [string]: default value (optional)
        * check [bool]: if False, value is not checked (optional, value=True)

    Color values are encoded as hexadecimal strings or Qt color names
    """

    def __init__(self, label, value=None, **kwargs):
        super().__init__(label, value=value, **kwargs)

    def _create_widget(self):
        widget = custom_widgets.ColorChooserButton()
        if self.value is not None:
            widget.set_value(self.value)
        return widget


class Enum(DataItem):
    """
    Construct a data item for a list of choices.
        * label [string]: name
        * choices [list, tuple or function]: string list or (key, label) list
          function of two arguments (item, value) returning a list of tuples
          (key, label, image) where image is an icon path, a QIcon instance
          or a function of one argument (key) returning a QIcon instance
        * value [-]: default label or default key (optional)
        * check [bool]: if False, value is not checked (optional, value=True)
        * radio [bool]: if True, shows radio buttons instead of a combo box
          (default is False)
    """

    def __init__(self,
                 label,
                 choices,
                 value=None,
                 radio=False,
                 **kwargs):
        super().__init__(label, value=value, **kwargs)
        self.radio = radio
        self.choices = choices

    def _create_widget(self):
        if self.radio:
            widget = custom_widgets.SelectionWidget(layout="vertical")
        else:
            widget = widgets.ComboBox()
        for item in self.choices:
            if isinstance(item, tuple):
                if len(item) == 2:
                    widget.add(item[1], item[0])
                elif len(item) == 3:
                    widget.add(item[1], item[0], item[2])
            else:
                widget.add(item)
        if self.value is not None:
            widget.set_value(self.value)
        return widget


class MultipleChoice(DataItem):

    def __init__(self, label, choices, value=None, **kwargs):
        super().__init__(label, value=value, **kwargs)
        self.choices = choices

    def _create_widget(self):
        widget = widgets.ListWidget()
        widget.set_selection_mode("multi")
        for item in self.choices:
            if isinstance(item, tuple):
                if len(item) == 2:
                    widget.add(item[1], item[0])
                elif len(item) == 3:
                    widget.add(item[1], item[0], item[2])
            else:
                widget.add(item)
        if self.value is not None:
            widget.set_value(self.value)
        return widget


class File(DataItem):

    def __init__(self, label, formats="*", value=None, save=True,
                 root=None, **kwargs):
        super().__init__(label, value=value, **kwargs)
        self.formats = formats.lstrip(".")
        self.root = root
        self.save = save

    def _create_widget(self):
        file_mode = "any_file" if self.save else "existing_file"
        mode = "save" if self.save else "open"
        widget = custom_widgets.FileChooserButton(file_mode=file_mode,
                                                  mode=mode,
                                                  root=self.root)
        if self.value is not None:
            widget.set_value(self.value)
        return widget


class Folder(DataItem):

    def __init__(self,
                 label,
                 value=None,
                 root=None,
                 mode="open",
                 **kwargs):
        super().__init__(label, value=value, **kwargs)
        self.mode = mode
        self.root = root

    def _create_widget(self):
        widget = custom_widgets.FileChooserButton(file_mode="directory",
                                                  mode=self.mode,
                                                  root=self.root)
        if self.value is not None:
            widget.set_value(self.value)
        return widget


class StringOrNumber(DataItem):

    def _create_widget(self):
        widget = custom_widgets.StringOrNumberWidget(self.label)
        if self.value is not None:
            widget.set_value(self.value)
        return widget


class Button(DataItem):
    """
    Construct a simple button that calls a method when hit
        * label [string]: text shown on the button
        * callback [function]: function with four params (dataset, item, value, parent)
            - dataset [DataSet]: instance of the parent dataset
            - item [DataItem]: instance of Button (i.e. self)
            - value [unspecified]: value of Button (default Button
              value or last value returned by the callback)
            - parent [QObject]: button's parent widget
        * icon [QIcon or string]: icon show on the button (optional)
        * check [bool]: if False, value is not checked (optional, value=True)
    """

    def __init__(self, label, callback, icon=None, value=None):
        super().__init__("", value=value, include=False)
        self.button_label = label
        self.icon = icon
        self.callback = callback

    def _create_widget(self):
        widget = widgets.PushButton(self.button_label)
        widget.set_icon(self.icon)
        if self.value is not None:
            widget.set_value(self.value)
        callback = functools.partial(self.callback, parent=widget.window())
        widget.clicked.connect(callback)
        return widget


class DataSetMeta(type):
    """
    DataSet metaclass

    Create class attribute `_items`: list of the DataSet class attributes,
    created in the same order as these attributes were written
    """
    def __new__(mcs, name, bases, dct):
        filtered = [b for b in bases if getattr(b, "__metaclass__", None) is DataSetMeta]
        items = {item._name: item for b in filtered for item in b._items}
        # items should contain DataItems of parent classes
        for attrname, value in list(dct.items()):
            if isinstance(value, DataItem):
                value.name = attrname
                items[attrname] = value
        dct["_items"] = items
        return type.__new__(mcs, name, bases, dct)


class DataSet(object, metaclass=DataSetMeta):

    def __init__(self, title=None, comment=None, icon=""):
        # self.widget = custom_widgets.SettingsWidget()
        self.dialog_title = title
        self.dialog_comment = comment
        self.dialog_icon = icon
        self.ok_btn = None

    def create_dialog(self):
        dialog = widgets.BaseDialog()
        dialog.set_modality("application")
        dialog.setMinimumWidth(400)
        dialog.title = self.dialog_title
        dialog.set_icon(self.dialog_icon)
        dialog.set_layout("grid")
        dialog.box.set_spacing(10)
        dialog.box.set_margin(20)

        button_box = widgets.DialogButtonBox()
        self.ok_btn = button_box.add_default_button("ok", callback=dialog.accept)
        button_box.add_default_button("cancel", callback=dialog.reject)
        widget_dict = {k: v.create_widget() for k, v in self._items.items()}
        widget_dict = {k: v for k, v in widget_dict.items() if v is not None}

        def on_update():
            is_valid = all(i.is_valid() if hasattr(i, "is_valid") else True
                           for i in widget_dict.values())
            self.ok_btn.setEnabled(is_valid)

        for i, (k, item) in enumerate(self._items.items()):
            if k not in widget_dict:
                continue
            dialog.box[i, item.label_col] = widgets.Label(item.label)
            dialog.box[i, item.label_col + 1:item.label_col + 3] = widget_dict[k]
            widget = widget_dict[k]
            widget.id = k
            widget.value_changed.connect(on_update)
            for active in item.enabled_on:
                widget.setEnabled(widget_dict[active].get_value())
                widget_dict[active].value_changed.connect(widget.setEnabled)
            for active in item.disabled_on:
                widget.setDisabled(widget_dict[active].get_value())
                widget_dict[active].value_changed.connect(widget.setDisabled)
        if self.dialog_comment:
            label = widgets.Label(self.dialog_comment)
            label.setWordWrap(True)
            dialog.box[len(self._items) + 1, 0:3] = label
        dialog.box.append(button_box)
        on_update()
        return dialog

    def edit(self, preset: Optional[dict] = None) -> bool:
        dialog = self.create_dialog()
        if preset:
            for item in dialog.layout():
                if item.id in preset and preset[item.id] is not None:
                    item.set_value(preset[item.id])

        if not dialog.show_blocking():
            return False
        new_values = {item.id: item.get_value()
                      for item in dialog.layout()
                      if item.id}
        enabled = {item.id: item.isEnabled()
                   for item in dialog.layout()
                   if item.id}
        # new_values = {a: (str(b) if isinstance(b, pathlib.Path) else b)
        #               for a, b in dct.items()}
        for k, item in self._items.items():
            if k in new_values:
                item.set_value(new_values[k])
                item.is_enabled = enabled[k]
        return True

    def to_dict(self) -> dict:
        return {k: (str(v.value) if isinstance(v.value, pathlib.Path) else v.value)
                for k, v in self._items.items()
                if v.include and v.is_enabled}


if __name__ == "__main__":
    app = widgets.app()

    class Test(DataSet):
        boolitem = Bool(label="Bool")
        string1 = String(label="String Regex 0-9")
        string2 = String(label="String Notempty")
        enum = Enum(label="Enum", choices=["A", "B"], disabled_on="boolitem")
        floatitem = Float(label="Float", enabled_on="boolitem")
        fileitem = File(label="File", optional="Activate")
        folderitem = Folder(label="Folder", include=False)
        buttonitem = Button(label="Folder", icon="mdi.folder", callback=print)
        stringornumber = StringOrNumber(label="StringOrNumber", value=2.4)
        code = Code(label="Test", value="class Test")
        # code = Regex(label="Test", value="[", show_error=False)

    # class Test2(Test):
    #     boolitem = None

    dlg = Test(icon="mdi.timer", comment="hallo")
    # dlg.widget.value_changed.connect(print)
    if dlg.edit(dict(boolitem=True)):
        from pprint import pprint
        pprint(dlg.to_dict())



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtGui, QtCore

from prettyqt.utils import bidict


SHAPES = bidict(arrow=QtCore.Qt.ArrowCursor,
                uparrow=QtCore.Qt.UpArrowCursor,
                cross=QtCore.Qt.CrossCursor,
                wait=QtCore.Qt.WaitCursor,
                caret=QtCore.Qt.IBeamCursor)


class Cursor(QtGui.QCursor):

    def set_shape(self, shape: str):
        """sets cursor shape

        Allowed values are "arrow", "uparrow", "cross", "wait", "caret"

        Args:
            shape: shape to use

        Raises:
            ValueError: shape does not exist
        """
        if shape not in SHAPES:
            raise ValueError("Invalid shape type.")
        self.setShape(SHAPES[shape])

    def get_shape(self) -> str:
        """returns current cursor shape

        Possible values: "arrow", "uparrow", "cross", "wait", "caret"

        Returns:
            cursor shape
        """
        return SHAPES.inv[self.shape()]



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
based on http://www.yasinuludag.com/blog/?p=49
"""

from prettyqt import core, gui, syntaxhighlighters

HIGHLIGHT_RULES = []


class Rule(syntaxhighlighters.HighlightRule):
    pass


class Element(Rule):
    regex = r"\b[A-Za-z0-9_]+(?=[\s/>])"
    color = "blue"


class Attribute(Rule):
    regex = r"\b[A-Za-z0-9_]+(?=\=)"
    color = "darkgreen"
    italic = True


class EqualSign(Rule):
    regex = r"="
    color = "darkgreen"


class LineComment(Rule):
    regex = r"<!--[^\n]*-->"
    color = "lightgrey"


class Text(Rule):
    regex = r">(.+)(?=</)"


class Keyword(Rule):
    regex = [r"\b?xml\b", "/>", ">", "<", "</"]
    bold = True
    color = "red"


VALUE_FORMAT = gui.TextCharFormat()
VALUE_FORMAT.set_foreground_color("orange")
VALUE_START_EXPRESSION = core.RegExp(r"\"")
VALUE_END_EXPRESSION = core.RegExp(r"\"(?=[\s></])")


class XmlHighlighter(gui.SyntaxHighlighter):

    RULES = Rule.__subclasses__()

    def highlightBlock(self, text):
        super().highlightBlock(text)
        # HANDLE QUOTATION MARKS NOW.. WE WANT TO START WITH " AND END WITH "..
        # A THIRD " SHOULD NOT CAUSE THE WORDS INBETWEEN SECOND AND THIRD
        # TO BE COLORED
        self.setCurrentBlockState(0)
        start_index = 0
        if self.previousBlockState() != 1:
            start_index = VALUE_START_EXPRESSION.indexIn(text)
        while start_index >= 0:
            end_index = VALUE_END_EXPRESSION.indexIn(text, start_index)
            if end_index == -1:
                self.setCurrentBlockState(1)
                comment_len = len(text) - start_index
            else:
                matched_len = VALUE_END_EXPRESSION.matchedLength()
                comment_len = end_index - start_index + matched_len
            self.setFormat(start_index, comment_len, VALUE_FORMAT)
            start_index = VALUE_START_EXPRESSION.indexIn(text, start_index + comment_len)


if __name__ == "__main__":
    from prettyqt import widgets
    app = widgets.app()
    editor = widgets.PlainTextEdit()
    highlighter = XmlHighlighter(editor.document())
    editor.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from typing import Callable, Optional

from qtpy import QtCore, QtWidgets

from prettyqt import core, widgets
from prettyqt.utils import bidict, icons


STYLES = bidict(icon=QtCore.Qt.ToolButtonIconOnly,
                text=QtCore.Qt.ToolButtonTextOnly,
                text_beside_icon=QtCore.Qt.ToolButtonTextBesideIcon,
                text_below_icon=QtCore.Qt.ToolButtonTextUnderIcon)

TOOLBAR_AREAS = bidict(left=QtCore.Qt.LeftToolBarArea,
                       right=QtCore.Qt.RightToolBarArea,
                       top=QtCore.Qt.TopToolBarArea,
                       bottom=QtCore.Qt.BottomToolBarArea,
                       all=QtCore.Qt.AllToolBarAreas,
                       none=QtCore.Qt.NoToolBarArea)


QtWidgets.QToolBar.__bases__ = (widgets.Widget,)


class ToolBar(QtWidgets.QToolBar):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.set_icon_size(24)
        self.menu_buttons = list()

    def add_menu_button(self,
                        label: str,
                        icon: icons.IconType,
                        menu: QtWidgets.QMenu) -> widgets.ToolButton:
        btn = widgets.ToolButton.for_menu(menu)
        btn.setText(label)
        btn.setToolButtonStyle(self.toolButtonStyle())
        btn.set_icon(icon)
        self.menu_buttons.append(btn)
        self.addWidget(btn)
        return btn

    def add_separator(self,
                      text: Optional[str] = None,
                      before: QtWidgets.QAction = None) -> QtWidgets.QAction:
        """adds a separator showing an optional label

        Args:
            text: Text to show on separator
            before: insert separator before specific action

        Returns:
            Separator action
        """
        if text is None:
            if before:
                return self.insertSeparator(before)
            else:
                return self.addSeparator()
        else:
            label = widgets.Label(text)
            label.setMinimumWidth(self.minimumWidth())
            label.setStyleSheet("background:lightgrey")
            label.set_alignment(horizontal="center")
            if before:
                return self.insertWidget(before, label)
            else:
                return self.addWidget(label)

    def set_style(self, style: str):
        if style is None:
            return None
        self.setToolButtonStyle(STYLES[style])
        for btn in self.menu_buttons:
            btn.set_style(style)

    def get_style(self) -> str:
        """returns current style

        Possible values: "icon", "text", "text_below_icon", "text_beside_icon"

        Returns:
            style
        """
        return STYLES.inv[self.toolButtonStyle()]

    def add_action(self,
                   label: str,
                   icon: icons.IconType = None,
                   callback: Optional[Callable] = None,
                   checkable: bool = False):
        icon = icons.get_icon(icon)
        action = self.addAction(icon, label)
        if callback is not None:
            action.triggered.connect(callback)
        if checkable:
            action.setCheckable(True)
        return action

    def add_spacer(self):
        spacer = widgets.Widget()
        spacer.set_size_policy("expanding", "expanding")
        return self.addWidget(spacer)

    def set_icon_size(self, size: int):
        self.setIconSize(core.Size(size, size))

    def set_font_size(self, size: int):
        self.setStyleSheet(f"font-size: {size}pt;")

    def is_area_allowed(self, area: str):
        """check if toolbar is allowed at specified area

        Valid values for area: "left", "right", "top", "bottom"

        Args:
            area: area of the toolbar

        Raises:
            ValueError: area does not exist
        """
        if area not in TOOLBAR_AREAS:
            raise ValueError("Area not existing")
        return self.isAreaAllowed(TOOLBAR_AREAS[area])

    def set_allowed_areas(self, areas):
        self.setAllowedAreas(TOOLBAR_AREAS[areas])


if __name__ == "__main__":
    app = widgets.app()
    widget = ToolBar()
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from typing import Optional, Tuple, List
import re

from qtpy import QtCore

from prettyqt import core
from prettyqt.utils import bidict

# IGNORECASE = QtCore.QRegularExpression.CaseInsensitiveOption
# MULTILINE = QtCore.QRegularExpression.MultilineOption
# DOTALL = QtCore.QRegularExpression.DotMatchesEverythingOption
# VERBOSE = QtCore.QRegularExpression.ExtendedPatternSyntaxOption

IGNORECASE = re.IGNORECASE
MULTILINE = re.MULTILINE
DOTALL = re.DOTALL
VERBOSE = re.VERBOSE

MAP = bidict({re.IGNORECASE: QtCore.QRegularExpression.CaseInsensitiveOption,
              re.MULTILINE: QtCore.QRegularExpression.MultilineOption,
              re.DOTALL: QtCore.QRegularExpression.DotMatchesEverythingOption,
              re.VERBOSE: QtCore.QRegularExpression.ExtendedPatternSyntaxOption})


def compile(pattern, flags=0):
    flag = QtCore.QRegularExpression.NoPatternOption
    for k, v in MAP.items():
        if k & flags:
            flag |= v
    return core.RegularExpression(pattern, flag)


def search(pattern, string, flags=0) -> Optional[core.RegularExpression]:
    compiled = compile(pattern, flags)
    return compiled.search(string)


def match(pattern, string, flags=0) -> Optional[core.RegularExpression]:
    compiled = compile(pattern, flags)
    return compiled.match(string)


def fullmatch(pattern, string, flags=0) -> Optional[core.RegularExpression]:
    compiled = compile(pattern, flags)
    return compiled.fullmatch(string)


# def split(pattern, string, maxsplit=0, flags=0) -> list:
#     compiled = compile(pattern, flags)
#     return compiled.split(string, maxsplit)


def findall(pattern, string, flags=0) -> List[str]:
    compiled = compile(pattern, flags)
    return compiled.findall(string)


def finditer(pattern, string, flags=0):
    compiled = compile(pattern, flags)
    return compiled.finditer(string)


def sub(pattern, repl, string, count=0, flags=0) -> str:
    compiled = compile(pattern, flags)
    return compiled.sub(repl, string, count)


def subn(pattern, repl, string, count=0, flags=0) -> Tuple[str, int]:
    compiled = compile(pattern, flags)
    return compiled.subn(repl, string, count)


def escape(pattern):
    dont_escape = ['!', '"', '%', "'", ',', '/', ':', ';', '<', '=', '>', '@', "`"]
    result = core.RegularExpression.escape(pattern)
    for i in dont_escape:
        result.replace(i, r"\i")
    assert result == re.escape(pattern)


if __name__ == "__main__":
    reg = compile("[0-9]+ [a-z] [0-9]+")



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtGui


class Palette(QtGui.QPalette):

    def highlight_inactive(self):
        color = self.color(self.Active, self.Highlight)
        self.setColor(self.Inactive, self.Highlight, color)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff

for full list, see:
- https://cdn.materialdesignicons.com/3.0.39/
"""

from typing import Union

from qtpy import QtGui, QtCore
from prettyqt import gui


ColorType = Union[str, int, QtGui.QRgba64, QtCore.Qt.GlobalColor,
                  QtGui.QColor, tuple, None]


def get_color(color: ColorType) -> gui.Color:
    if isinstance(color, (tuple, list)):
        return gui.Color(*color)
    return gui.Color(color)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from typing import Union
import pathlib

from qtpy import QtWebEngineWidgets

from prettyqt import core, widgets

QtWebEngineWidgets.QWebEngineView.__bases__ = (widgets.Widget,)


class WebEngineView(QtWebEngineWidgets.QWebEngineView):

    @classmethod
    def from_local_file(cls, path):
        url = core.Url.fromLocalFile(str(path))
        reader = cls()
        reader.setUrl(url)
        return reader

    def set_url(self, url: Union[str, pathlib.Path]):
        if isinstance(url, pathlib.Path):
            url = core.Url.fromLocalFile(str(url))
        elif isinstance(url, str):
            url = core.Url(url)
        self.setUrl(url)

    def load_url(self, url: Union[str, pathlib.Path]):
        if isinstance(url, pathlib.Path):
            url = core.Url.fromLocalFile(str(url))
        elif isinstance(url, str):
            url = core.Url(url)
        self.load(url)

    def set_zoom(self, zoom: float):
        self.setZoomFactor(zoom)


if __name__ == "__main__":
    from prettyqt import widgets
    app = widgets.app()
    path = path = "E:\\dev\\datacook\\processanalyzer\\docs\\index.html"
    widget = WebEngineView.from_local_file(path)
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""


from qtpy import QtCore, QtWidgets

from prettyqt import core
from prettyqt.utils import bidict, icons

PRIORITIES = bidict(low=QtWidgets.QAction.LowPriority,
                    normal=QtWidgets.QAction.NormalPriority,
                    high=QtWidgets.QAction.HighPriority)

CONTEXTS = bidict(widget=QtCore.Qt.WidgetShortcut,
                  widget_with_children=QtCore.Qt.WidgetWithChildrenShortcut,
                  window=QtCore.Qt.WindowShortcut,
                  application=QtCore.Qt.ApplicationShortcut)


QtWidgets.QAction.__bases__ = (core.Object,)


class Action(QtWidgets.QAction):

    def __init__(self, parent=None,
                 text: str = "",
                 icon: icons.IconType = None,
                 shortcut: str = "",
                 tooltip: str = ""):
        super().__init__(parent)
        self.set_text(text)
        self.set_icon(icon)
        self.set_shortcut(shortcut)
        self.set_tooltip(tooltip)

    def __getstate__(self):
        return dict(text=self.text(),
                    enabled=self.isEnabled(),
                    shortcut=self.shortcut(),
                    tooltip=self.toolTip(),
                    checkable=self.isCheckable(),
                    checked=self.isChecked(),
                    priority=self.get_priority(),
                    shortcut_context=self.get_shortcut_context(),
                    statustip=self.statusTip())

    def __setstate__(self, state):
        self.__init__()
        self.set_text(state.get("text", ""))
        self.set_enabled(state.get("enabled", True))
        self.set_shortcut(state["shortcut"])
        self.set_tooltip(state.get("tooltip", ""))
        self.set_statustip(state.get("statustip", ""))
        self.set_checked(state.get("checked", False))
        self.set_priority(state["priority"])
        self.set_shortcut_context(state["shortcut_context"])
        self.set_checkable(state["checkable"])

    def set_text(self, text: str):
        self.setText(text)

    def set_enabled(self, enabled: bool = True):
        self.setEnabled(enabled)

    def set_disabled(self):
        self.setEnabled(False)

    def set_tooltip(self, text: str):
        self.setToolTip(text)

    def set_statustip(self, text: str):
        self.setStatusTip(text)

    def set_checked(self, value: bool):
        self.setChecked(value)

    def set_checkable(self, value: bool):
        self.setCheckable(value)

    def set_icon(self, icon: icons.IconType):
        """set the icon for the action

        Args:
            icon: icon to use
        """
        icon = icons.get_icon(icon)
        self.setIcon(icon)

    def set_shortcut(self, shortcut):
        if shortcut:
            self.setShortcut(shortcut)

    def set_menu(self, menu):
        self.setMenu(menu)

    def set_priority(self, priority: str):
        """set priority of the action

        Allowed values are "low", "normal", "high"

        Args:
            priority: priority for the action

        Raises:
            ValueError: priority does not exist
        """
        if priority not in PRIORITIES:
            raise ValueError(f"{priority} not a valid priority.")
        self.setPriority(PRIORITIES[priority])

    def get_priority(self) -> str:
        """returns current priority

        Possible values: "low", "normal", "high"

        Returns:
            priority
        """
        return PRIORITIES.inv[self.priority()]

    def set_shortcut_context(self, context: str):
        """set shortcut context

        Allowed values are "widget", "widget_with_children", "window", "application"

        Args:
            context: shortcut context

        Raises:
            ValueError: shortcut context does not exist
        """
        if context not in CONTEXTS:
            raise ValueError(f"{context} not a valid shortcut context.")
        self.setShortcutContext(CONTEXTS[context])

    def get_shortcut_context(self) -> str:
        """returns shortcut context

        Possible values: "widget", "widget_with_children", "window", "application"

        Returns:
            shortcut context
        """
        return CONTEXTS.inv[self.shortcutContext()]

    def show_shortcut_in_contextmenu(self, state: bool = True):
        self.setShortcutVisibleInContextMenu(state)


if __name__ == "__main__":
    from prettyqt import widgets
    app = widgets.app()
    action = Action("This is a test")
    print(action.__getstate__())
    app.exec_()



# -*- coding: utf-8 -*-

"""charts module
"""

from .abstractseries import AbstractSeries
from .xyseries import XYSeries
from .scatterseries import ScatterSeries
from .lineseries import LineSeries
from .chart import Chart
from .chartview import ChartView

__all__ = ["AbstractSeries",
           "XYSeries",
           "ChartView",
           "Chart",
           "LineSeries",
           "ScatterSeries"]
